/*  This file is part of distributed convex BP - http://www.alexander-schwing.de/
 *
 *  distributed convex BP is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  distributed convex BP is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with distributed convex BP.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2010-2011  Alexander Schwing  [aschwing _at_ inf _dot_ ethz _dot_ ch]
 */

#ifndef __CLIENT_HEADERS__
#define __CLIENT_HEADERS__

#include "GeneralDefinitions.h"

template <class T>
class Client {
	int ClientID;

	size_t num_Nodes;
	size_t num_Factors;
	size_t num_minor_iterations;

	size_t* s_nodePotentialIX;
	size_t* s_NodePotPosition;
	T* s_NodePotentials;

	size_t* s_factorPotentialIX;
	size_t* s_FactorPotPosition;
	T* s_FactorPotentials;

	size_t* CumPosNodesToFactors;
	size_t* CumPosFactorsToNodes;
	size_t* CumPosMsg;
	size_t* Nalpha;
	size_t* Ni;
	size_t* NodePosInFactor;

	size_t* s_FactorVariableSizePosition;
	size_t* s_FactorVariableSize;

	size_t* s_NodePotSize;
	size_t* s_DividedFactors;

	T* log_n;
	T* log_ns;
	size_t* CumPosFactorBelief;
	size_t* CumPosNodeBelief;

	T agreementThreshold;
	T epsilon;
	T* hat_c_alpha;
	T* c_i;

	size_t num_agrees;

	std::vector<std::vector<size_t> > Colors;
	
	T computePrimal(T* localBelief);
	T computeDual();
	T computeM(size_t i, size_t alphaIX, size_t x_i);
	void computeN(size_t i);
	void computeLocalBelief(T* localBelief);
	void computeFactorBelief(T* factorBelief);
	void checkAgreement(bool* agreement, T* factorBelief, T* localBelief);

public:
	Client();
	~Client();

	int ReceiveProblem(int myid);
	int DoComputation();
	int SendMsg();
	int ReceiveUpdateOrBeliefRequest();
	int ComputeAndSendLocalBeliefs();
	int ProcessUpdate();
	int GreedyColoring();

};

#endif