/*  This file is part of distributed convex BP - http://www.alexander-schwing.de/
 *
 *  distributed convex BP is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  distributed convex BP is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with distributed convex BP.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2010-2011  Alexander Schwing  [aschwing _at_ inf _dot_ ethz _dot_ ch]
 */

#ifndef __GENERAL_DEFINITIONS__
#define __GENERAL_DEFINITIONS__
#include <cstddef>
#define _SECURE_SCL 1		//default is one

#include <vector>		//for size_t

//use size_t if OpenMP supports unsigned iteration variables on your platform
#if defined __WINDOWS__ || defined __MAC__
	typedef int	OPENMP_ITER_TYPE;
#else
	typedef size_t	OPENMP_ITER_TYPE;
#endif

//we need a definite 64bit unsigned integer
#if defined __WINDOWS__
	typedef unsigned __int64 uint64;
#else
	typedef unsigned long long uint64;
#endif

template <class T>
struct PotentialStruct {
	T* pot;
	size_t* num_states;
	size_t num_variables;
	PotentialStruct() : pot(0), num_states(0), num_variables(-1) {}
	PotentialStruct(T* p, size_t* num_s, int num_v) : pot(p), num_states(num_s), num_variables(num_v) {}
};

template <class T>
struct cBP_Settings {
	size_t ServerIterations;
	size_t ClientIterations;
	T epsilon;
	T agreementThreshold;
	char* fn;
	char* OutputFile;
	cBP_Settings() : ServerIterations(5), ClientIterations(1), epsilon(T(1.0)), agreementThreshold(T(1e-5)), fn(0), OutputFile(0) {}
};

#ifdef __WINDOWS__
#include <windows.h>
#else
#include <sys/time.h>
#endif

class CPrecisionTimer
{
#ifdef __WINDOWS__
	LARGE_INTEGER lFreq, lStart;
#else
	timeval lStart;
#endif

public:
	CPrecisionTimer()
	{
#ifdef __WINDOWS__
		QueryPerformanceFrequency(&lFreq);
#endif
	}

	inline void Start()
	{
#ifdef __WINDOWS__
		QueryPerformanceCounter(&lStart);
#else
		gettimeofday(&lStart, 0);
#endif
	}

	inline double Stop()
	{
		// Return duration in seconds...
#ifdef __WINDOWS__
		LARGE_INTEGER lEnd;
		QueryPerformanceCounter(&lEnd);
		return (double(lEnd.QuadPart - lStart.QuadPart) / lFreq.QuadPart);
#else
		timeval lFinish;
		gettimeofday(&lFinish, 0);
		return double(lFinish.tv_sec - lStart.tv_sec) + double(lFinish.tv_usec - lStart.tv_usec)/1e6; 
#endif
	}
};

#ifdef min
#undef min
#endif
#ifdef max
#undef max
#endif

#endif