function [rows, cols, states, numMachines] = CBPTsukuba(fnleft, fnright, states)

if nargin<3
    fnleft = 'scene1.row3.col4_small.ppm';
    fnright = 'scene1.row3.col3_small.ppm';
    states = 8;
end

imleft = 255*im2double(imread(fnleft));
imright = 255*im2double(imread(fnright));

imleft = imleft(:,1:end-states+1,:);

rows = size(imleft,1);
cols = size(imleft,2);

SD = zeros([size(imleft,1),size(imleft,2),states]);

for k=1:states
    SD(:,:,k) = sum((imright(:,k:end-states+k,:) - imleft).^2,3);
end

% [~,IX] = min(SD,[],3);
% IX = IX - 1;
% figure;imshow(IX./max(IX(:)));

SD = -SD/2000;

num_nodes = size(SD,1)*size(SD,2);
num_cliques = size(SD,1)*(size(SD,2)-1) + size(SD,2)*(size(SD,1)-1);

vars = reshape(0:num_nodes-1,[size(SD,1),size(SD,2)]);
cliques = zeros(num_cliques, 4);
n1 = vars(1:end-1,:);
n2 = vars(2:end,:);
cliques(1:size(SD,2)*(size(SD,1)-1),:) = [ones(numel(n1),1)*[2 num_nodes] n1(:) n2(:)];
n1 = vars(:,1:end-1);
n2 = vars(:,2:end);
cliques(size(SD,2)*(size(SD,1)-1)+1:end,:) = [ones(numel(n1),1)*[2 num_nodes] n1(:) n2(:)];

SD = permute(SD,[3,1,2]);
SD = reshape(SD,[size(SD,1),size(SD,2)*size(SD,3)]);

[X Y] = meshgrid(1:states, 1:states);
reg = 5-min(abs(X-Y),5);

accuracy = 8;                                                               %4: single precision (float); 8: double precision (double)

% problem divided onto two clients
fc = floor(cols/2);
machineIDs = [zeros(rows, fc) ones(rows, cols-fc)];

numMachines = max(machineIDs(:))+1;                                         %machineID starts with zero

fid = fopen('../tsukuba.cbp','wb');
fwrite(fid, accuracy, 'uint64');                                            %double or single precision accuracy; 
fwrite(fid, num_nodes, 'uint64');                                           %number of variables/nodes
fwrite(fid, states*ones(num_nodes,1), 'uint64');                            %cardinality of variables
fwrite(fid, machineIDs(:), 'uint64');                                       %machineID of variables
if(accuracy==4)                                                             %c_i
    fwrite(fid, ones(num_nodes,1), 'float32');
else
    fwrite(fid, ones(num_nodes,1), 'double');
end
fwrite(fid,num_nodes + num_cliques,'uint64');                               %number of potentials, i.e. factors of any kind
fwrite(fid,[ones(1,num_nodes);0:num_nodes-1;0:num_nodes-1],'uint64');       %node potential specification {Cliquesize=1; PotentialIX; VariableID}
fwrite(fid,cliques.','uint64');                                             %clique potentials specification {Cliquesize; PotentialIX; VariableIDs}
if(accuracy==4)                                                             %c_alpha for all clique potentials, i.e. those potentials with CliqueSize > 1
    fwrite(fid,ones(1,num_cliques),'float32');
else
    fwrite(fid,ones(1,num_cliques),'double');
end
for k=1:num_nodes                                                           %potentials, i.e. {PotentialIX, PotentialValues (note that format is different from UAI (Matlab format)!!!)...}
    fwrite(fid,k-1,'uint64');           %potentialIX
    if(accuracy==4)
        fwrite(fid,SD(:,k),'float32');  %PotentialValues
    else
        fwrite(fid,SD(:,k),'double');   %PotentialValues
    end
end
fwrite(fid,num_nodes,'uint64');         %potentialIX
if(accuracy==4)
    fwrite(fid,reg(:),'float32');       %PotentialValues
else
    fwrite(fid,reg(:),'double');        %PotentialValues
end
fclose(fid);
