function [rows, cols, states] = UAITsukuba(fnleft, fnright, states)

if nargin<3
    fnleft = 'scene1.row3.col4_small.ppm';
    fnright = 'scene1.row3.col3_small.ppm';
    states = 8;
end

imleft = 255*im2double(imread(fnleft));
imright = 255*im2double(imread(fnright));

imleft = imleft(:,1:end-states+1,:);

rows = size(imleft,1);
cols = size(imleft,2);

SD = zeros([size(imleft,1),size(imleft,2),states]);

for k=1:states
    SD(:,:,k) = sum((imright(:,k:end-states+k,:) - imleft).^2,3);
end

% [~,IX] = min(SD,[],3);
% IX = IX - 1;
% figure;imshow(IX./max(IX(:)));

SD = -SD/2000;

num_nodes = size(SD,1)*size(SD,2);
num_cliques = size(SD,1)*(size(SD,2)-1) + size(SD,2)*(size(SD,1)-1);

vars = reshape(0:num_nodes-1,[size(SD,1),size(SD,2)]);
cliques = zeros(num_cliques, 3);
n1 = vars(1:end-1,:);
n2 = vars(2:end,:);
cliques(1:size(SD,2)*(size(SD,1)-1),:) = [2*ones(numel(n1),1) n1(:) n2(:)];
n1 = vars(:,1:end-1);
n2 = vars(:,2:end);
cliques(size(SD,2)*(size(SD,1)-1)+1:end,:) = [2*ones(numel(n1),1) n1(:) n2(:)];

SD = permute(SD,[3,1,2]);
SD = reshape(SD,[size(SD,1),size(SD,2)*size(SD,3)]);

fmt = '%d ';
fmt = [num2str(states),' ',repmat(fmt,1,states),'\n'];

[X Y] = meshgrid(1:states, 1:states);
reg = 5-min(abs(X-Y),5);
regstr = [num2str(states*states), ' ', num2str(reg(:).'),'\n'];
regstr = repmat(regstr,num_cliques,1).';

fid = fopen('../tsukuba.uai','wt');
fprintf(fid,'MARKOV\n');
fprintf(fid,'%d\n', num_nodes);
fprintf(fid,'%d ', states*ones(num_nodes,1));
fprintf(fid,'\n%d\n', num_nodes + num_cliques);
fprintf(fid,'%d %d\n', [ones(1,num_nodes);0:num_nodes-1]);
fprintf(fid,'%d %d %d\n', cliques.');
fprintf(fid,fmt,SD);
fprintf(fid,regstr);
fclose(fid);
