/*  This file is part of distributed convex BP - http://www.alexander-schwing.de/
 *
 *  distributed convex BP is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  distributed convex BP is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with distributed convex BP.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2010-2011  Alexander Schwing  [aschwing _at_ inf _dot_ ethz _dot_ ch]
 */

#ifndef __SERVER_HEADERS__
#define __SERVER_HEADERS__

#include "GeneralDefinitions.h"

#include <map>
#include <set>

template <class T>
class Server {
	size_t m_numMachines;
	size_t m_numNodes;
	struct cBP_Settings<double>* cBPparams;

	//initial variables obtained from user
	std::vector<std::vector<size_t> >* nodeTo;				
	std::vector<std::vector<size_t> >* factorTo;				
	std::vector<PotentialStruct<T> >* nodePotentials;	
	std::vector<PotentialStruct<T> >* factorPotentials;
	std::vector<size_t>* nodePotentialIX;					
	std::vector<size_t>* factorPotentialIX;
	std::vector<size_t>* NodesOnMachine;
	std::vector<size_t>* variableSize;
	std::vector<T>* in_c_i;
	std::vector<T>* in_c_alpha;
	
	std::vector<std::set<size_t> > Npalpha;

	T dualValue, primalValue, num_agreements;

	//
	std::vector<std::vector<size_t> > AllS_DividedFactors;
	std::vector<std::map<size_t, size_t> > AllS_OldFactorIndexToNew;
	std::vector<std::vector<size_t> > AllS_CumPosFactorsToNodes;
	std::vector<std::vector<size_t> > AllS_Nalpha;
	std::vector<std::vector<size_t> > AllS_NodePotSize;
	std::vector<std::vector<size_t> > AllS_NodePosInFactor;
	std::vector<std::map<size_t, size_t> > AllS_OldNodeIndexToNew;

public:
	Server(std::vector<std::vector<size_t> >* NToF, std::vector<std::vector<size_t> >* FToN, std::vector<PotentialStruct<T> >* nodePotentials, std::vector<PotentialStruct<T> >* factorPotentials, std::vector<size_t>* nodePotentialIX, std::vector<size_t>* factorPotentialIX, std::vector<size_t>* variableSize, struct cBP_Settings<double>* params, std::vector<size_t>* NodesOnMachine, std::vector<T>* c_in, std::vector<T>* c_alpha);
	~Server();

	int SendProblemToClient(size_t k);
	int NewIteration();
	int ReceiveMsgFromClient(int machine, std::vector<char*>& IdxMsg, std::vector<int>& IdxSz, std::vector<char*>& MsgMsg);
	int ComputePrimalDualAgreement();
	int ComputeAndSendUpdates(std::vector<char*>& IdxMsg, std::vector<int>& IdxSz, std::vector<char*>& MsgMsg);
	int SendBeliefRequest();
	int ReceiveBeliefs(const char* fn);
};

#endif